-- ################### --
-- Minimap Coordinates
-- ################### --
local coordFS = Minimap:CreateFontString("FontString","OVERLAY","GameTooltipText")
	coordFS:SetTextColor(1, 1, 1)

local function coordRefresh()
	local uiMapID = C_Map.GetBestMapForUnit("player")
	if not uiMapID then return end
	local mapInfo = C_Map.GetMapInfo(uiMapID)
	if not mapInfo.mapID then coordFS:SetText("--, --") return end
	local pos = C_Map.GetPlayerMapPosition(mapInfo.mapID, "player")
	if (pos == nil) or (pos.x == nil) or (pos.y == nil) then
		coordFS:SetText("--, --")
		return
	end
	if pos and pos.x ~= 0 and pos.y ~= 0 then
		coordFS:SetFormattedText("%.1f, %.1f", pos.x*100, pos.y*100)
	end
end

local timeElapsed = 0
Minimap:HookScript("OnUpdate", function(self, elapsed)
	local inInstance, instanceType = IsInInstance()
	if not IsInInstance(player) then
		timeElapsed = timeElapsed + elapsed
		if timeElapsed >= 1 then
			coordRefresh()
			timeElapsed = 0
		end
	else
		coordFS:SetFormattedText("")
	end
end)

-- ################################# --
-- MinimapCluster Mouseover Function --
-- ################################# --
local defaultScale = MINIMAP_BOTTOM_EDGE_EXTENT or 192 -- 192 is the default height as per Blizzard's comment: "pixels from the top of the screen to the bottom edge of the minimap, needed for UIParentManageFramePositions"

local DMM_SHOW_BUTTONS = {
	MinimapCluster.IndicatorFrame.MailFrame,
	MinimapCluster.InstanceDifficulty,
	AddonCompartmentFrame,
	GameTimeFrame,
	TimeMagerClockButton,
	MinimapCluster.Selection,
	Minimap.ZoomHitArea,
	Minimap.ZoomIn,
	Minimap.ZoomOut,
	ExpansionLandingPageMinimapButton,
	Minimap,
	MinimapCluster.BorderTop,
	MinimapCluster.MinimapContainer,
	MinimapCluster.TrackingFrame.Button,
	MinimapCluster.TrackingFrame,
	MinimapCluster.ZoneTextButton,
	MinimapCluster,
}

local DMM_HIDE_BUTTONS = {
	AddonCompartmentFrame,
	GameTimeFrame,
	MinimapCluster.TrackingFrame,
}

local DVMMBAlphaTimer

local function showMMB()
	if DVMMBAlphaTimer then DVMMBAlphaTimer:Cancel() end
    for _, v in pairs(DMM_SHOW_BUTTONS) do
        v:SetAlpha(1)
    end
	MinimapCluster.ZoneTextButton:SetPoint("LEFT", MinimapCluster.TrackingFrame, "RIGHT", 0, 0)
end

local function hideMMB()
    for _, v in pairs(DMM_HIDE_BUTTONS) do
        v:SetAlpha(0)
    end
	MinimapCluster.ZoneTextButton:SetPoint("LEFT", MinimapCluster.BorderTop, "LEFT", 0, 0)
end

local function delayHideMMB()
	DVMMBAlphaTimer = C_Timer.NewTimer(0.75, hideMMB)
end

for _, v in pairs(DMM_SHOW_BUTTONS) do
    v:HookScript("OnEnter", showMMB)
    v:HookScript("OnLeave", delayHideMMB)
end

local function setHeadUnder()
	if MinimapCluster.BorderTop then
		MinimapCluster.BorderTop:ClearAllPoints()
		MinimapCluster.BorderTop:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT", 0, 0)
		MinimapCluster.BorderTop:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 0, 0)
	end

	if TimeManagerClockButton then
		TimeManagerClockTicker:ClearAllPoints()
		TimeManagerClockTicker:SetPoint("BOTTOM", Minimap, "BOTTOM", 0, MinimapCluster.ZoneTextButton:GetHeight())
	end

	if MinimapCluster.IndicatorFrame then
		MinimapCluster.IndicatorFrame:ClearAllPoints()
		MinimapCluster.IndicatorFrame:SetPoint("BOTTOMLEFT", MinimapCluster.BorderTop, "TOPLEFT", 0, 0)
	end

	if Minimap.ZoomIn then
		Minimap.ZoomIn:ClearAllPoints()
		Minimap.ZoomIn:SetPoint("BOTTOMRIGHT", Minimap, "RIGHT", 0, -( Minimap.ZoomIn:GetWidth() ))
	end

	if coordFS then
		coordFS:SetPoint("TOP", Minimap, "TOP", 0, -2)
	end
end

local function DMM_Square()
	local headerUnderneath = MinimapCluster:GetSettingValueBool(Enum.EditModeMinimapSetting.HeaderUnderneath)
	local size = ( MinimapCluster:GetHeight() )
	local scale = ( size/defaultScale )

	MinimapCluster:SetSize(size, size)

	Minimap:SetParent(MinimapCluster)
	Minimap:ClearAllPoints()
	Minimap:SetAllPoints(MinimapCluster)
	Minimap:SetSize(size, size)

	-- Minimap Texture Mask --
	Minimap:SetMaskTexture('Interface\\ChatFrame\\ChatFrameBackground')

	-- Compass Texture --
	MinimapCompassTexture:Hide()

	-- Coordinate Font Setting and Scaling --
	coordFS:SetFont("Fonts\\FRIZQT__.TTF", 14*scale, "OUTLINE")
	coordFS:SetPoint("TOP", Minimap, "TOP", 0, -( MinimapCluster.BorderTop:GetHeight() )*scale)

	if MinimapCluster.BorderTop then
		if MinimapCluster.TrackingFrame then
			MinimapCluster.TrackingFrame:ClearAllPoints()
			MinimapCluster.TrackingFrame:SetPoint("TOPLEFT", MinimapCluster.BorderTop, "TOPLEFT", 0, 0)
			MinimapCluster.TrackingFrame:SetScale(scale)
		end

		if GameTimeFrame then
			GameTimeFrame:ClearAllPoints()
			GameTimeFrame:SetPoint("RIGHT", MinimapCluster.BorderTop, "RIGHT", 0, 0)
			GameTimeFrame:SetScale(scale)
		end

		if 	MinimapCluster.IndicatorFrame then
			MinimapCluster.IndicatorFrame:ClearAllPoints()
			MinimapCluster.IndicatorFrame:SetScale(scale)
			MinimapCluster.IndicatorFrame:SetPoint("TOPLEFT", MinimapCluster.BorderTop, "BOTTOMLEFT", 0, 0)
		end

		MinimapCluster.BorderTop:ClearAllPoints()
		MinimapCluster.BorderTop:SetWidth(size)
		MinimapCluster.BorderTop:SetScale(scale)
		MinimapCluster.BorderTop:SetPoint("TOPLEFT", Minimap, "TOPLEFT", 0, 0)
		MinimapCluster.BorderTop:SetPoint("TOPRIGHT", Minimap, "TOPRIGHT", 0, 0)
	end

	if MinimapCluster.MinimapContainer then
		MinimapCluster.MinimapContainer:ClearAllPoints()
		MinimapCluster.MinimapContainer:SetAllPoints(MinimapCluster)
	end

	if MinimapBackdrop then
		MinimapBackdrop:ClearAllPoints()
		MinimapBackdrop:SetAllPoints(MinimapCluster)
	end

	if Minimap.ZoomIn then
		Minimap.ZoomIn:ClearAllPoints()
		Minimap.ZoomIn:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 0, Minimap.ZoomIn:GetWidth())
	end

	if Minimap.ZoomOut then
		Minimap.ZoomOut:ClearAllPoints()
		Minimap.ZoomOut:SetPoint("TOPRIGHT", Minimap.ZoomIn, "BOTTOMLEFT", 0, 0)
	end

	if MinimapCluster.ZoneTextButton then
		MinimapCluster.ZoneTextButton:ClearAllPoints()
		MinimapCluster.ZoneTextButton:SetScale(scale)
		hideMMB()
	end

	if TimeManagerClockButton then
		TimeManagerClockButton:ClearAllPoints()
		TimeManagerClockButton:SetPoint("BOTTOM", Minimap, "BOTTOM", 0, 0)
		TimeManagerClockButton:SetScale(scale*1.4)

		TimeManagerClockTicker:ClearAllPoints()
		TimeManagerClockTicker:SetPoint("BOTTOM", Minimap, "BOTTOM", 0, 2)
		TimeManagerClockTicker:SetFont("Fonts\\FRIZQT__.TTF", select(2, TimeManagerClockTicker:GetFont()), "OUTLINE")
	end

	if ExpansionLandingPageMinimapButton then
		ExpansionLandingPageMinimapButton:ClearAllPoints()
		ExpansionLandingPageMinimapButton:SetPoint("CENTER", Minimap, "LEFT", 0, 0)
		ExpansionLandingPageMinimapButton:SetScale(scale)
	end

	-- if MinimapCluster.Selection then
	-- 	MinimapCluster.Selection:ClearAllPoints()
	-- 	MinimapCluster.Selection:SetPoint("TOPLEFT", MinimapCluster, "TOPLEFT", 2, -2)
	-- 	MinimapCluster.Selection:SetPoint("BOTTOMRIGHT", MinimapCluster, "BOTTOMRIGHT", -2, 2)
	-- end
	-- Hack to make the damned thing update with each slider size value change when in Edit Mode.
	local zoom = Minimap:GetZoom() -- Get current zoom
	Minimap:SetZoom(5) -- Zoom out to cache the unzoomed minimap
	Minimap:SetZoom(0) -- Zoom out to cache the unzoomed minimap
	Minimap:SetZoom(zoom) -- Set the zoom back to what it was at

	if (headerUnderneath) then
		setHeadUnder()
	end
end

-- ################################# --
-- Resize Minimap and Make it Square --
-- ################################# --
MinimapCluster:RegisterEvent("ADDON_LOADED")
MinimapCluster:RegisterEvent("PLAYER_LOGIN")
MinimapCluster:RegisterEvent("PLAYER_ENTERING_WORLD")
MinimapCluster:RegisterEvent("ZONE_CHANGED");
MinimapCluster:RegisterEvent("ZONE_CHANGED_INDOORS");
MinimapCluster:RegisterEvent("ZONE_CHANGED_NEW_AREA");
MinimapCluster:RegisterEvent("EDIT_MODE_LAYOUTS_UPDATED");

-- MinimapCluster:SetClampRectInsets(0, 0, 0, 0)
-- MinimapCluster:SetClampedToScreen(true)

MinimapCluster:HookScript("OnEvent", function(self, event, ...)
	DMM_Square()
end)

hooksecurefunc(EditModeSystemSettingsDialog, "OnSettingValueChanged", DMM_Square)
